# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging

import six
from six import BytesIO, StringIO

from driverng import MessageV2, Worklist
from driverng.exceptions import ParametrageError
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.thermofisher")


class ThermofisherBase(Splitted):

    SEPARATOR = ","
    ENCODING = "utf-8"


class ThermofisherResult(object):
    FIELDS = [
        String("Well"),
        String("WellPosition"),
        String("Omit"),
        String("Sample"),
        String("Target"),
        String("Task"),
        String("Reporter"),
        String("Quencher"),
        String("AmpStatus"),
        String("AmpScore"),
        String("CurveQuality"),
        String("ResultQualityIssues"),
        String("Cq"),
        String("CqConfidence"),
        String("CqMean"),
        String("CqSD"),
        String("AutoThreshold"),
        String("Threshold"),
        String("AutoBaseline"),
        String("BaselineStart"),
        String("BaselineEnd"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            setattr(self, field.name, line[index])


class ThermofisherResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: ThermofisherResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.date_resu = None
        self.instrument_serial_number = None
        for row in reader:
            if six.PY2:
                line = [
                    c.decode("utf-8") for c in row
                ]  # PY2 csv reader est imcompatible avec unicode
            else:
                line = row
            if line[0].startswith("#"):
                if "Plate Run End Date/Time:" in line[0]:
                    try:
                        self.date_resu = datetime.datetime.strptime(
                            line[0].split(":", 1)[1].strip()[:19], "%Y-%m-%d %H:%M:%S"
                        )
                    except ValueError as e:
                        logger.info(
                            "La date des résultat n'est pas au bon format %s", e
                        )
                if "Instrument Serial Number:" in line[0]:
                    self.instrument_serial_number = line[0].split(":")[1].strip()
                continue
            if not len(line):
                continue
            if len(line) == len(ThermofisherResult.FIELDS):
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "thermofisher", "label": "Thermofisher QS5 "}
    OPTIONS = [
        MessageV2.Option("analyseur_traites", label="Liste des analyseurs traités")
    ]

    def __init__(self, **kwargs):
        analyseur_traites = kwargs.pop("analyseur_traites", None)
        self.analyseur_traites = (
            analyseur_traites.split(",") if analyseur_traites is not None else None
        )
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = ThermofisherResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if (
            self.analyseur_traites is not None
            and message.instrument_serial_number not in self.analyseur_traites
        ):
            raise ParametrageError(
                "l'instrument %s n'est pas traité par la connexion"
                % message.instrument_serial_number
            )
        for m in message:
            if not m.Target and not m.Sample and not m.Cq:
                continue
            elif m.Sample == "Sample" and m.Well == "Well" and m.Cq == "Cq":
                continue
            sample = worklist.get_or_create(
                sample_id=m.Sample,
                rack="{}-{}".format(m.Well, m.WellPosition),
                type="patient",
            )
            for attr in ["Cq", "CqConfidence", "CqMean", "CqSD"]:
                resu = getattr(m, attr).replace(",", ".").strip()
                if resu:
                    sample.results.add(
                        code="{}_{}".format(m.Target, attr),
                        value=getattr(m, attr).replace(",", "."),
                        instrument=message.instrument_serial_number,
                        date=message.date_resu,
                    )
        return worklist
